@@ -1,3 +1,18 @@
+/* Copyright 2001-2004 The Apache Software Foundation
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
 #include "mod_perl.h"
 
 /*
@@ -49,6 +64,9 @@ static void modperl_bucket_sv_destroy(void *data)
 
 static const apr_bucket_type_t modperl_bucket_sv_type = {
     "mod_perl SV bucket", 4,
+#if MODULE_MAGIC_NUMBER >= 20020602
+    APR_BUCKET_DATA,
+#endif
     modperl_bucket_sv_destroy,
     modperl_bucket_sv_read,
     apr_bucket_setaside_notimpl,
